/*======================================================================
	See GNU.txt, CREDITS.txt, and COPYING.txt
	Quake owned by ID Software, Inc.
	Code based on FitzQuake (props to John Fitzgibbons)
	QuakeCPP by Brian "Feared" Wychopen II
======================================================================*/
// quakedef.h -- primary header for client

//#define	GLTEST			// experimental stuff

#ifndef QUAKEDEF_H
#define QUAKEDEF_H

#define	QUAKE_GAME			// as opposed to utilities

#define	VERSION				1.09
#define	GLQUAKE_VERSION		1.00
#define	D3DQUAKE_VERSION	0.01
#define	WINQUAKE_VERSION	0.996
#define	LINUX_VERSION		1.30
#define	X11_VERSION			1.10

#define	FITZQUAKE_VERSION	0.85 //johnfitz

//define	PARANOID			// speed sapping error checking

#define	GAMENAME	"id1"		// directory to look in by default

#include <math.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>
#include <assert.h> //johnfitz

#if defined(_WIN32) && !defined(WINDED)

#if defined(_M_IX86)
#define __i386__	1
#endif

void	VID_LockBuffer (void);
void	VID_UnlockBuffer (void);

#else

#define	VID_LockBuffer()
#define	VID_UnlockBuffer()

#endif

#if defined __i386__ // && !defined __sun__
#define id386	1
#else
#define id386	0
#endif

#if id386
#define UNALIGNED_OK	1	// set to 0 if unaligned accesses are not supported
#else
#define UNALIGNED_OK	0
#endif

// !!! if this is changed, it must be changed in d_ifacea.h too !!!
#define CACHE_SIZE	32		// used to align key data structures

#define UNUSED(x)	(x = x)	// for pesky compiler / lint warnings

#define	MINIMUM_MEMORY			0x550000
#define	MINIMUM_MEMORY_LEVELPAK	(MINIMUM_MEMORY + 0x100000)

#define MAX_NUM_ARGVS	50

// up / down
#define	PITCH	0

// left / right
#define	YAW		1

// fall over
#define	ROLL	2


#define	MAX_QPATH		64			// max length of a quake game pathname
#define	MAX_OSPATH		128			// max length of a filesystem pathname

#define	ON_EPSILON		0.1			// point on plane side epsilon

#define	MAX_MSGLEN			32000	// max length of a reliable message //johnfitz -- was 8000
#define	MAX_DATAGRAM		32000	// max length of unreliable message //johnfitz -- was 1024

#define DATAGRAM_MTU		1400	// johnfitz -- actual limit for unreliable messages to nonlocal clients

//
// per-level limits
//

#define MIN_EDICTS		256			// johnfitz -- lowest allowed value for max_edicts cvar
#define MAX_EDICTS		32000		// johnfitz -- highest allowed value for max_edicts cvar
									// ents past 8192 can't play sounds in the standard protocol
#define	MAX_LIGHTSTYLES	64
#define	MAX_MODELS		2048		// johnfitz -- was 256
#define	MAX_SOUNDS		1024		// johnfitz -- was 256

#define	SAVEGAME_COMMENT_LENGTH	39

#define	MAX_STYLESTRING	64

//
// stats are integers communicated to the client by the server
//
#define	MAX_CL_STATS		32
#define	STAT_HEALTH			0
#define	STAT_FRAGS			1
#define	STAT_WEAPON			2
#define	STAT_AMMO			3
#define	STAT_ARMOR			4
#define	STAT_WEAPONFRAME	5
#define	STAT_SHELLS			6
#define	STAT_NAILS			7
#define	STAT_ROCKETS		8
#define	STAT_CELLS			9
#define	STAT_ACTIVEWEAPON	10
#define	STAT_TOTALSECRETS	11
#define	STAT_TOTALMONSTERS	12
#define	STAT_SECRETS		13		// bumped on client side by svc_foundsecret
#define	STAT_MONSTERS		14		// bumped by svc_killedmonster

// stock defines

#define	IT_SHOTGUN				1
#define	IT_SUPER_SHOTGUN		2
#define	IT_NAILGUN				4
#define	IT_SUPER_NAILGUN		8
#define	IT_GRENADE_LAUNCHER		16
#define	IT_ROCKET_LAUNCHER		32
#define	IT_LIGHTNING			64
#define IT_SUPER_LIGHTNING      128
#define IT_SHELLS               256
#define IT_NAILS                512
#define IT_ROCKETS              1024
#define IT_CELLS                2048
#define IT_AXE                  4096
#define IT_ARMOR1               8192
#define IT_ARMOR2               16384
#define IT_ARMOR3               32768
#define IT_SUPERHEALTH          65536
#define IT_KEY1                 131072
#define IT_KEY2                 262144
#define	IT_INVISIBILITY			524288
#define	IT_INVULNERABILITY		1048576
#define	IT_SUIT					2097152
#define	IT_QUAD					4194304
#define IT_SIGIL1               (1<<28)
#define IT_SIGIL2               (1<<29)
#define IT_SIGIL3               (1<<30)
#define IT_SIGIL4               (1<<31)

//===========================================
//rogue changed and added defines

#define RIT_SHELLS              128
#define RIT_NAILS               256
#define RIT_ROCKETS             512
#define RIT_CELLS               1024
#define RIT_AXE                 2048
#define RIT_LAVA_NAILGUN        4096
#define RIT_LAVA_SUPER_NAILGUN  8192
#define RIT_MULTI_GRENADE       16384
#define RIT_MULTI_ROCKET        32768
#define RIT_PLASMA_GUN          65536
#define RIT_ARMOR1              8388608
#define RIT_ARMOR2              16777216
#define RIT_ARMOR3              33554432
#define RIT_LAVA_NAILS          67108864
#define RIT_PLASMA_AMMO         134217728
#define RIT_MULTI_ROCKETS       268435456
#define RIT_SHIELD              536870912
#define RIT_ANTIGRAV            1073741824
#define RIT_SUPERHEALTH         2147483648

//MED 01/04/97 added hipnotic defines
//===========================================
//hipnotic added defines
#define HIT_PROXIMITY_GUN_BIT 16
#define HIT_MJOLNIR_BIT       7
#define HIT_LASER_CANNON_BIT  23
#define HIT_PROXIMITY_GUN   (1<<HIT_PROXIMITY_GUN_BIT)
#define HIT_MJOLNIR         (1<<HIT_MJOLNIR_BIT)
#define HIT_LASER_CANNON    (1<<HIT_LASER_CANNON_BIT)
#define HIT_WETSUIT         (1<<(23+2))
#define HIT_EMPATHY_SHIELDS (1<<(23+3))

//===========================================

#define	MAX_SCOREBOARD		16
#define	MAX_SCOREBOARDNAME	32

#define	SOUND_CHANNELS		8

#include "common.h"
#include "bspfile.h"
#include "vid.h"
#include "sys.h"
#include "zone.h"
#include "mathlib.h"

typedef struct
{
	vec3_t			origin;
	vec3_t			angles;
	unsigned short 	modelindex;	//johnfitz -- was int
	unsigned short 	frame;		//johnfitz -- was int
	unsigned char 	colormap;	//johnfitz -- was int
	unsigned char 	skin;		//johnfitz -- was int
	unsigned char	alpha;		//johnfitz -- added
	int				effects;
} entity_state_t;

#include "wad.h"
#include "draw.h"
#include "cvar.h"
#include "screen.h"
#include "net.h"
#include "protocol.h"
#include "cmd.h"
#include "sbar.h"
#include "sound.h"
#include "render.h"
#include "client.h"
#include "progs.h"
#include "server.h"
#include "renderer\gl_model.h"

#include "image.h"
#include "renderer\gl_texmgr.h"

#include "input.h"
#include "world.h"
#include "keys.h"
#include "console.h"
#include "view.h"
#include "menu.h"
#include "crc.h"
#include "cdaudio.h"
#include "renderer\glquake.h"


//=============================================================================

// the host system specifies the base of the directory tree, the
// command line parms passed to the program, and the amount of memory
// available for the program to use

typedef struct
{
	char	*basedir;
	char	*cachedir;		// for development over ISDN lines
	int		argc;
	char	**argv;
	void	*membase;

	// Feared; int to SIZE_T
	SIZE_T	memsize;
} quakeparms_t;

//=============================================================================

extern bool noclip_anglehack;

// host
extern	quakeparms_t	host_parms;

extern	cvar_t	sys_ticrate;
extern	cvar_t	sys_nostdout;
extern	cvar_t	developer;
extern	cvar_t	max_edicts; //johnfitz

extern	bool	host_initialized;	// true if into command execution
extern	double	host_frametime;
extern	byte	*host_colormap;
extern	int		host_framecount;	// incremented every frame, never reset
extern	double	realtime;			// not bounded in any way, changed at
									// start of every frame, never reset

void Host_ClearMemory(void);
void Host_ServerFrame(void);
void Host_InitCommands(void);
void Host_Init(quakeparms_t *parms);
void Host_Shutdown(void);
void Host_Error(char *error, ...);
void Host_EndGame(char *message, ...);
void Host_Frame(float time);
void Host_Quit_f(void);
void Host_ClientCommands(char *fmt, ...);
void Host_ShutdownServer(bool crash);

extern bool		msg_suppress_1;		// suppresses resolution and cache size console output
									//  an fullscreen DIB focus gain/loss
extern int		current_skill;		// skill level for currently loaded level (in case
									//  the user changes the cvar while the level is
									//  running, this reflects the level actually in use)

extern bool		isDedicated;
extern SIZE_T	minimum_memory;

extern void Host_WriteConfiguration(void); //

// chase
extern	cvar_t	chase_active;

extern void Chase_Init (void);
extern void Chase_UpdateForClient (void);	//johnfitz
extern void Chase_UpdateForDrawing (void);	//johnfitz

extern void TraceLine(vec3_t start, vec3_t end, vec3_t impact);

// world
extern bool SV_RecursiveHullCheck(hull_t *hull, int num, float p1f, float p2f, vec3_t p1, vec3_t p2, trace_t *trace);
extern void R_MarkSurfaces(void);
extern void R_CullSurfaces(void);
extern void R_UpdateWarpTextures(void);
extern void R_UpdateWarpTextures(void);
extern void R_DrawTextureChains_ShowTris(void);
extern void R_DrawWorld(void);
extern void R_DrawTextureChains_Water(void);

// console
extern char *Con_Quakebar(int len);
extern void Con_LogCenterPrint(char *str);
extern void Con_TabComplete(void);

// gl_sky
extern void Sky_LoadSkyBox(char *name);
extern void Sky_LoadTexture(texture_t *mt);
extern void Sky_DrawSky(void);
extern void Sky_Init(void);
extern void Sky_NewMap(void);

// common
extern void COM_CreatePath(char *path);

// gl_screen
extern void SCR_LoadPics(void);

// sbar
extern void Sbar_LoadPics(void);

// warp
extern void GL_SubdivideSurface(msurface_t *fa);
extern void DrawWaterPoly(glpoly_t *p);
extern int gl_warpimagesize; //johnfitz -- for water warp

// mesh
extern void GL_MakeAliasModelDisplayLists(model_t *, aliashdr_t *);

// rlight
extern void R_AnimateLight(void);
extern void R_RenderDlights(void);
extern int R_LightPoint(vec3_t p);
extern void R_MarkLights(dlight_t *light, int bit, mnode_t *node);

// view
extern void V_CalcBlend(void);
extern void V_UpdateBlend(void);

// alias
extern void R_DrawAliasModel(entity_t *);
extern void R_DrawAliasModel_ShowTris(entity_t *);
extern void GL_DrawAliasShadow(entity_t *);

// brush
extern void R_DrawBrushModel(entity_t *e);
extern void R_DrawBrushModel_ShowTris(entity_t *e);
extern void R_RebuildAllLightmaps(void);
extern void GL_BuildLightmaps(void);
extern void DrawGLPoly(glpoly_t *p);
extern void R_RenderDynamicLightmaps(msurface_t *fa);
extern void DrawGLTriangleFan(glpoly_t *p);
extern void R_UploadLightmap(int lmap);

// sprite
extern void R_DrawSpriteModel(entity_t *e);

// part
extern void R_DrawParticles_ShowTris(void);
extern void R_DrawParticles(void);
extern void R_InitParticles(void);
extern void R_ClearParticles(void);
extern void CL_RunParticles(void);

// model
extern void Mod_SetExtraFlags(model_t *mod);

// draw
extern void GL_Set2D(void);
extern void Draw_NewGame(void);
extern void GL_SetCanvas(canvastype newcanvas);

// main
extern bool R_CullModelForEntity(entity_t *e);
extern void R_RotateForEntity(vec3_t origin, vec3_t angles);
extern bool R_CullBox(vec3_t emins, vec3_t emaxs);

// texmgr
extern void TexMgr_RecalcWarpImageSize(void);

// cd_wjn
extern LONG CDAudio_MessageHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

// cvar
extern void Cvar_Init(void);

//host_cmd
extern void ExtraMaps_Init(void);
extern void Modlist_Init(void);

// misc
extern void R_NewGame(void);

// gl_refrag
extern void R_StoreEfrags(efrag_t **ppefrag);

// vidnt
extern void M_Print(int cx, int cy, char *str);
extern viddef_t vid;	// global video state

#endif
